@@ -4,7 +4,7 @@
 #       CIPP::DB_DBI
 #
 # REVISION
-#	$Revision: 1.6 $
+#	$Revision: 1.7 $
 #
 # METHODEN
 #       siehe CIPP_DB.interface
@@ -89,7 +89,7 @@
 package CIPP::DB_DBI;
 
 $VERSION = "0.51";
-$REVISION = q$Revision: 1.6 $;
+$REVISION = q$Revision: 1.7 $;
 
 sub new {
 	my ($type) = shift;
@@ -365,34 +365,4 @@ sub Get_DB_Handle {
 	return $code;
 }
 
-
-
 1;
-__END__
-
-=head1 NAME
-
-CIPP::DB_DBI - CIPP database module to generate DBI code
-
-=head1 DESCRIPTION
-
-CIPP has a database code abstraction layer, so it can
-generate code to access databases via different interfaces.
-
-This module is used by CIPP to generate code to access
-databases via DBI, version >= 0.93.
-
-=head1 AUTHOR
-
-Jrn Reder, joern@dimedis.de
-
-=head1 COPYRIGHT
-
-Copyright 1997-1999 dimedis GmbH, All Rights Reserved
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
-
-=head1 SEE ALSO
-
-perl(1), CIPP (3pm)
