@@ -4,7 +4,7 @@
 #	CIPP::Runtime.pm
 #
 # REVISION
-#	$Revision: 1.2 $
+#	$Revision: 1.5 $
 #
 # DESCRIPTION
 #	Enthlt Funktionen, die zur Laufzeit von CIPP-CGI Programmen
@@ -123,7 +123,7 @@
 
 package CIPP::Runtime;
 
-$REVISION = q$Revision: 1.2 $;
+$REVISION = q$Revision: 1.5 $;
 $VERSION = "0.31";
 
 use FileHandle;
@@ -132,7 +132,7 @@ use Cwd;
 sub Read_Config {
 	my ($filename, $nocache) = @_;
 
-	die "CONFIG\tDatei '$filename' nicht gefunden" if not -f $filename;
+	die "CONFIG\File '$filename' not found" if not -f $filename;
 	
 	my $file_timestamp = (stat($filename))[9];
 	
@@ -162,22 +162,17 @@ sub Exception {
 
 	my $log_error = Log ("EXC", "TYPE=$type, MESSAGE=$message");
 	if ( $log_error ) {
-		$message .= "<P><BR><B>Diese Exception konnte nicht im Logfile vermerkt werden!</B><BR>\n";
+		$message .= "<P><BR><B>Unable to add this exception to the logfile!</B><BR>\n";
 		$message .= "=> $log_error";
 	}
 	print "Content-type: text/html\n\n" if ! $CIPP_Exec::cipp_http_header_printed;
 	print "<P>$CIPP_Exec::cipp_error_text<P>";
 
 	if ( $CIPP_Exec::cipp_error_show ) {
-		$message =~ s/\n/<BR><BR>/;
-		$message =~ s/\n/<BR>/g;
-		$message =~ s/ at /<P>at /;
-	
 		print "<P><B>EXCEPTION: </B>$type<BR>\n",
-		      "<B>MESSAGE: </B><BR><BLOCKQUOTE><TT>$message</TT></BLOCKQUOTE><P>\n";
+		      "<B>MESSAGE: </B>$message<P>\n";
 		if ( $message =~ /compilation errors/ ) {
-			print "<P>Die Compiler-Fehlermeldung finden Sie im Logfile des\n";
-			print "Webservers.<P>\n";
+			print "<P>You will find the compiler error messages in the webserver error log<P>\n";
 		}
 	}
 
@@ -218,11 +213,11 @@ sub Log {
 
 		if ( open (cipp_LOG_FILE, ">> $filename") ) {
 			if ( ! print cipp_LOG_FILE "$time\t$msg\n" ) {
-				$log_error = "Konnte nicht in $filename schreiben";
+				$log_error = "Can't write data to '$filename'";
 			}
 			close cipp_LOG_FILE;
 		} else {
-			$log_error = "Konnte $filename nicht zum Schreiben ffnen";
+			$log_error = "Can't write '$filename'";
 		}
 	} else {
 		$CIPP_Exec::apache_request->log_error ("Log: $msg");
@@ -275,14 +270,14 @@ sub Execute {
 
 	my $cwd_dir = cwd();
 	chdir $dir
-		or die "$throw\tKonnte nicht nach Verzeichnis $dir wechseln";
+		or die "$throw\tUnable to chdir to '$dir'";
 
 	# CGI-Script einlesen
 
 	my $cgi_fh = new FileHandle;
 	if ( ! open ($cgi_fh, $script) ) {
 		chdir $cwd_dir;
-		die "$throw\tKonnte '$script' nicht ffnen";
+		die "$throw\tUnable to open '$script'";
 	}
 
 	my $cgi_script = join ("", <$cgi_fh>);
@@ -293,7 +288,7 @@ sub Execute {
 	my $save_fh = "save".(++$CIPP::Runtime::save_stdout);
 	if ( ! open ($save_fh, ">&STDOUT") ) {
 		chdir $cwd_dir;
-		die "$throw\tKonnte STDOUT nicht duplizieren";
+		die "$throw\tUnable to dup STDOUT";
 	}
 
 	# Dateinamen fr Ausgabe ermitteln:
@@ -318,10 +313,10 @@ sub Execute {
 	close STDOUT;
 	if ( ! open (STDOUT, "> $catch_file") ) {
 		open (STDOUT, ">&$save_fh")
-			or die "$throw\tKonnte STDOUT nicht restaurieren";
+			or die "$throw\tUnable to restore STDOUT";
 		close $save_fh;
 		chdir $cwd_dir;
-		die "$throw\tKonnte '$catch_file' nicht zum Schreiben ffnen";
+		die "$throw\tCan't write '$catch_file'";
 	}
 
 	# Lschen des Error-Handlers und Setzen der Variablen
@@ -347,7 +342,7 @@ sub Execute {
 
 	close STDOUT;
 	open (STDOUT, ">&$save_fh")
-		or die "$throw\tKonnte STDOUT nicht restaurieren";
+		or die "$throw\tUnable to restore STDOUT";
 	close $save_fh;
 
 	# Wenn Ergebnis in Variable soll, machen wir's doch
@@ -356,11 +351,11 @@ sub Execute {
 	if ( ref ($output) eq 'SCALAR' ) {
 		my $catch_fh = new FileHandle;
 		open ($catch_fh, $catch_file)
-			or die "$throw\tFehler beim Einlesen der Scriptausgabe";
+			or die "$throw\tError reading the script output";
 		$$output = join ("", <$catch_fh>);
 		close $catch_fh;
 		unlink $catch_file
-			or die "$throw\tFehler beim Lschen der Datei '$catch_file'";
+			or die "$throw\tError deleting file '$catch_file': $!";
 	}
 
 
@@ -419,9 +414,9 @@ sub Get_Object_URL {
 	# wenn nicht eindeutig: Fehler!
 
 	if ( scalar @filenames == 0 ) {
-		die "$throw\tKonnte Objekt '$object_name' nicht auflsen";
+		die "$throw\tUnable to resolve object '$object_name'";
 	} elsif ( scalar @filenames > 1 ) {
-		die "$throw\tObject-Name '$object_name' ist nicht eindeutig";
+		die "$throw\tObject identifier '$object_name' is ambiguous";
 	}
 
 	my $file = $filenames[0];
@@ -430,30 +425,4 @@ sub Get_Object_URL {
 	return "$CIPP_Exec::cipp_doc_url/$file";
 }
 
-
-
 1;
-__END__
-
-=head1 NAME
-
-CIPP::Runtime - Runtime library for CIPP generated perl programs
-
-=head1 DESCRIPTION
-
-This module is used by Perl programs which are generated by CIPP.
-
-=head1 AUTHOR
-
-Jrn Reder, joern@dimedis.de
-
-=head1 COPYRIGHT
-
-Copyright 1997-1999 dimedis GmbH, All Rights Reserved
-
-This library is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
-
-=head1 SEE ALSO
-
-perl(1), CIPP (3pm)
